package com.tanpu.fund.api;

import com.tanpu.common.model.Page;
import com.tanpu.common.model.Pageable;
import com.tanpu.common.model.product.req.ProductListReq;
import com.tanpu.common.model.product.resp.*;
import com.tanpu.common.model.user.resp.SysConstantResp;
import com.tanpu.common.resp.CommonResp;
import com.tanpu.fund.api.model.req.FundQaAskReq;
import com.tanpu.fund.api.model.req.FundQaListReq;
import com.tanpu.fund.api.model.req.NetReq;
import com.tanpu.fund.api.model.req.ProductInfoReq;
import com.tanpu.fund.api.model.resp.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;
import java.util.List;

@Api(tags = "产品服务")
public interface ProductApi {

    /**
     * /sys/constant tamp_user 中的接口
     * 策略需要维护到sys_constant中
     */
    @ApiOperation("根据条件获取探普产品信息")
    @PostMapping("/get/product/info")
    CommonResp<Page<ProductInfoVO>> getProductInfo(@ApiParam("筛选条件对象") @RequestBody ProductInfoReq req);

    @ApiOperation("根据条件获取非探普产品信息")
    @PostMapping("/not/tanpu/product/info")
    CommonResp<Page<ProductInfoVO>> getNotTanpuProductInfo(@ApiParam("筛选条件对象") @RequestBody ProductInfoReq req);

    @ApiOperation("获取产品详情")
    @GetMapping("/product/detail")
    CommonResp<ProductInfoVO> getProductDetailInfo(@ApiParam("产品id") @RequestParam("id") String id);

    @ApiOperation("单位净值")
    @GetMapping("/unit/net")
    CommonResp<UnitNetVO> getUnitNetInfo(@ApiParam("产品id") @RequestParam("id") String id, @ApiParam("对比基金id") @RequestParam(value = "ratioId", required = false) String ratioId,
                                         @ApiParam("查询时间段 1:近1个月 2:近3个月 3:近6个月 4:近1年 99:成立以来") @RequestParam("type") String type);

    @ApiOperation("收益走势")
    @GetMapping("/profit/trend")
    CommonResp<UnitNetVO> getProfitTrend(@ApiParam("产品id") @RequestParam("id") String id, @ApiParam("对比基金id") @RequestParam(value = "ratioId", required = false) String ratioId,
                                         @ApiParam("查询时间段 1:近1个月 2:近3个月 3:近6个月 4:近1年 98:自定义 99:成立以来") @RequestParam("type") String type,
                                         @ApiParam("开始时间") @RequestParam(value = "beginTime", required = false) Long beginTime,
                                         @ApiParam("开始时间") @RequestParam(value = "endTime", required = false) Long endTime);

    @ApiOperation("净值列表查询")
    @PostMapping("/get/net/list")
    CommonResp<Page<NetVO>> getNetListInfo(@ApiParam("净值列表筛选对象") @RequestBody NetReq req);

    @ApiOperation("分红配送")
    @GetMapping("/bonus/ratio")
    CommonResp<List<BonusRatioVO>> getBonusRatioInfo(@ApiParam("产品id") @RequestParam("id") String id);

    @ApiOperation("历史业绩")
    @GetMapping("/track/record")
    CommonResp<List<TrackRecordVO>> getTrackRecordInfo(@ApiParam("产品id") @RequestParam("id") String id);

    @ApiOperation("历史净值")
    @GetMapping("/track/net")
    CommonResp<Page<TrackNetVO>> getTrackNetListInfo(@ApiParam("产品id") @RequestParam("id") String id, @ApiParam("分页对象") Pageable page);

    @ApiOperation("风险评估")
    @GetMapping("/get/risk/rating")
    CommonResp<List<RiskRatingVO>> getRiskRatingInfo(@ApiParam("产品id") @RequestParam("id") String id);

    @ApiOperation("基金经理")
    @GetMapping("/fund/manager")
    CommonResp<List<FundManagerVO>> getFundManagerInfo(@ApiParam("产品id") @RequestParam("id") String id);

    @ApiOperation("基金档案")
    @GetMapping("/fund/record")
    CommonResp<FundRecordVO> getFundRecordInfo(@ApiParam("产品id") @RequestParam("id") String id);

    @ApiOperation("基金公司")
    @GetMapping("/fund/company")
    CommonResp<FundCompanyVO> getFundCompanyInfo(@ApiParam("基金公司id") @RequestParam("id") String id);

    @ApiOperation("产品QA提问")
    @PostMapping("/QA/ask")
    CommonResp<QAVO> productQaAsk(@ApiParam("问题内容") @RequestBody FundQaAskReq fundQaAskReq);

    @ApiOperation("产品QA列表查询")
    @PostMapping("/get/product/QA")
    CommonResp<Page<QAVO>> getProductQAInfo(@ApiParam("产品QA条件信息") @RequestBody FundQaListReq fundQaAskReq);

    @ApiOperation("组合列表")
    @GetMapping("/fund/set/list")
    CommonResp<Page<FundSetListVO>> getFundSetList(@ApiParam("策略类型") @RequestParam("strategyType") Integer strategyType, @ApiParam("分页信息") Pageable page);

    @ApiOperation("组合详情")
    @GetMapping("/fund/group/detail")
    CommonResp<FundGroupVO> getFundGroupDetail(@ApiParam("组合id") @RequestParam("id") String id);

    @ApiOperation("历史持仓日期")
    @GetMapping("/track/position/dates")
    CommonResp<List<Long>> getTrackPositionDates(@ApiParam(value = "基金ID") @RequestParam("id") String id);

    @ApiOperation("历史持仓")
    @GetMapping("/track/position")
    CommonResp<TrackPositionVO> getTrackPositionDetail(@ApiParam("产品id") @RequestParam("id") String id, @ApiParam("日期") @RequestParam("date") Long date);

    @ApiOperation("净值数据导入")
    @PostMapping("/net/excel")
    CommonResp exportNetExcel(@RequestParam("url") String url);

    @ApiOperation(value = "根据产品查询标签")
    @GetMapping(value = "/h5/tag/querybyrelid", produces = {"application/json"})
    CommonResp<List<ProductLabel>> querytagbyrelid(@RequestParam("relId") String relId);

    @ApiOperation("产品解读")
    @GetMapping("/explain")
    CommonResp<List<FundExplainVO>> getFundExplainInfo(@RequestParam("id") String id);

    @ApiOperation("附件查询")
    @GetMapping("/business/attachment")
    CommonResp<List<AttachmentInfoVO>> getAttachmentInfo(@ApiParam("业务id") @RequestParam("id") String id,
                                                         @ApiParam("0 查询所有 1:公告,2:附件,3:探普研报4:赎回指南") @RequestParam("type") Integer type);

    @ApiOperation("查询产品策略类型")
    @GetMapping("/get/product/queryByconstantGroup")
    CommonResp<List<SysConstantResp>> getProductTypes(@ApiParam(value = "产品类型 tampstrategy:探普产品 ppstrategy:私募排排产品 tampstrategy:自建产品") @RequestParam("constantGroup") String constantGroup);

    @ApiOperation("根据条件获取探普产品信息")
    @PostMapping("/get/product/list")
    CommonResp<List<ProductInfoVO>> getProductInfolist(@ApiParam("筛选条件对象") @RequestBody ProductListReq req);

    @ApiOperation("根据条件获取非探普产品信息")
    @PostMapping("/not/tanpu/product/list")
    CommonResp<List<ProductInfoVO>> getNotTanpuProductInfolist(@ApiParam("筛选条件对象") @RequestBody ProductListReq req);

    @ApiOperation("动态回撤")
    @GetMapping("/dynamic/retreat")
    CommonResp<DynamicRetreatVO> getDynamicRetreatInfo(@RequestParam("id") String id);

    @ApiOperation("开放日赎回日查询")
    @GetMapping("/fund/date")
    CommonResp<FundDateResp> getFundDateByFundIdInfo(
            @ApiParam(value = "基金id", required = true) @Valid @NotEmpty(message = "基金id不能为空") @RequestParam("fundId") String fundId);
}
