package com.tanpu.fund.controller;

import com.tanpu.common.auth.mapping.TanpuInterfaceLoginAuth;
import com.tanpu.common.model.Page;
import com.tanpu.common.model.Pageable;
import com.tanpu.common.model.product.req.ProductListReq;
import com.tanpu.common.model.product.resp.*;
import com.tanpu.common.model.user.resp.SysConstantResp;
import com.tanpu.common.resp.CommonResp;
import com.tanpu.fund.api.ProductApi;
import com.tanpu.fund.api.model.req.FundQaAskReq;
import com.tanpu.fund.api.model.req.FundQaListReq;
import com.tanpu.fund.api.model.req.NetReq;
import com.tanpu.fund.api.model.req.ProductInfoReq;
import com.tanpu.fund.api.model.resp.*;
import com.tanpu.fund.feign.user.FeignClientForFatools;
import com.tanpu.fund.service.FundCombineInfoService;
import com.tanpu.fund.service.FundHistoryHoldService;
import com.tanpu.fund.service.FundQAService;
import com.tanpu.fund.service.ProductService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;

@RestController
@Slf4j
public class ProductController implements ProductApi {

    @Resource
    private FundHistoryHoldService fundHistoryHoldService;
    @Resource
    private ProductService productService;
    @Resource
    private FundCombineInfoService fundCombineInfoService;
    @Resource
    private FundQAService fundQAService;

    @Resource
    private FeignClientForFatools feignClientForFatools;

    @Override
    public CommonResp<Page<ProductInfoVO>> getProductInfo(ProductInfoReq req) {
        return CommonResp.success(this.productService.getProductList(req));
    }

    @Override
    public CommonResp<Page<ProductInfoVO>> getNotTanpuProductInfo(ProductInfoReq req) {
        return CommonResp.success(this.productService.getNotTanpuProduct(req));
    }

    @Override
    public CommonResp<ProductInfoVO> getProductDetailInfo(String id) {
        return CommonResp.success(this.productService.getProductDetail(id));
    }

    @Override
    public CommonResp<UnitNetVO> getUnitNetInfo(String id, String ratioId, String type) {
        return CommonResp.success(this.productService.getUnitNet(id, ratioId, type));
    }

    @Override
    public CommonResp<UnitNetVO> getProfitTrend(String id, String ratioId, String type, Long beginTime, Long endTime) {
        return CommonResp.success(this.productService.getProfitTrend(id, ratioId, type, beginTime, endTime));
    }

    @Override
    public CommonResp<Page<NetVO>> getNetListInfo(NetReq req) {
        return CommonResp.success(this.productService.getNetList(req));
    }

    @Override
    public CommonResp<List<BonusRatioVO>> getBonusRatioInfo(String id) {
        return CommonResp.success(this.productService.getBonusRatio(id));
    }

    @Override
    public CommonResp<List<TrackRecordVO>> getTrackRecordInfo(String id) {
        return CommonResp.success(this.productService.getTrackRecord(id));
    }

    @Override
    public CommonResp<Page<TrackNetVO>> getTrackNetListInfo(String id, Pageable page) {
        return CommonResp.success(this.productService.getTrackNetList(id, page));
    }

    @Override
    public CommonResp<List<RiskRatingVO>> getRiskRatingInfo(String id) {
        return CommonResp.success(this.productService.getRiskRating(id));
    }

    @Override
    public CommonResp<List<FundManagerVO>> getFundManagerInfo(String id) {
        return CommonResp.success(this.productService.getFundManager(id));
    }

    @Override
    public CommonResp<FundRecordVO> getFundRecordInfo(String id) {
        return CommonResp.success(this.productService.getFundRecord(id));
    }

    @Override
    public CommonResp<FundCompanyVO> getFundCompanyInfo(String id) {
        return CommonResp.success(this.productService.getFundCompany(id));
    }

    @Override
    @TanpuInterfaceLoginAuth
    public CommonResp<QAVO> productQaAsk(FundQaAskReq fundQaAskReq) {
        if (StringUtils.isEmpty(fundQaAskReq.getFundId())
                || StringUtils.isEmpty(fundQaAskReq.getContent())) {
            return CommonResp.error(CommonResp.PARAMETER_INVALID_STATUS_CODE, CommonResp.PARAMETER_INVALID_MESSAGE);
        }

        QAVO qavo = this.fundQAService.productQaAsk(fundQaAskReq);
        return CommonResp.success(qavo);
    }

    @Override
    public CommonResp<Page<QAVO>> getProductQAInfo(FundQaListReq fundQaAskReq) {
        Page<QAVO> productQAInfo = this.fundQAService.getProductQAInfo(fundQaAskReq.getFundId(), fundQaAskReq.getPage());
        return CommonResp.success(productQAInfo);
    }

    @Override
    public CommonResp<Page<FundSetListVO>> getFundSetList(Integer strategyType, Pageable page) {
        if (strategyType == null) {
            return CommonResp.error(CommonResp.PARAMETER_INVALID_STATUS_CODE, CommonResp.PARAMETER_INVALID_MESSAGE);
        }

        Page<FundSetListVO> fundSetListVOPage = this.fundCombineInfoService.queryFundCombines(strategyType, page);
        return CommonResp.success(fundSetListVOPage);
    }


    @Override
    public CommonResp<FundGroupVO> getFundGroupDetail(String id) {
        FundGroupVO detail = this.fundCombineInfoService.detail(id);
        return CommonResp.success(detail);
    }

    @Override
    public CommonResp<List<Long>> getTrackPositionDates(String id) {
        List<Long> trackPositionDates = this.fundHistoryHoldService.getTrackPositionDates(id);
        return CommonResp.success(trackPositionDates);
    }

    @Override
    public CommonResp<TrackPositionVO> getTrackPositionDetail(String id, Long date) {
        TrackPositionVO trackPositionDetail = this.fundHistoryHoldService.getTrackPositionDetail(id, date);
        return CommonResp.success(trackPositionDetail);
    }

    @Override
    public CommonResp exportNetExcel(String url) {
        if (StringUtils.isEmpty(url)) {
            return CommonResp.error();
        } else {

            return CommonResp.success();
        }
    }

    @Override
    public CommonResp<List<ProductLabel>> querytagbyrelid(String relId) {
        return this.feignClientForFatools.querytagbyrelid(relId);
    }

    @Override
    public CommonResp<List<FundExplainVO>> getFundExplainInfo(String id) {
        return CommonResp.success(this.productService.getFundExplain(id));
    }

    @Override
    public CommonResp<List<AttachmentInfoVO>> getAttachmentInfo(String id, Integer type) {
        return CommonResp.success(this.productService.getAttachment(id, type));
    }

    @Override
    public CommonResp<List<SysConstantResp>> getProductTypes(String constantGroup) {
        return this.feignClientForFatools.queryLabels(constantGroup);
    }

    @Override
    public CommonResp<List<ProductInfoVO>> getProductInfolist(ProductListReq req) {
        List<ProductInfoVO> productInfolist = this.productService.getProductInfolist(req);
        return CommonResp.success(productInfolist);
    }

    @Override
    public CommonResp<List<ProductInfoVO>> getNotTanpuProductInfolist(ProductListReq req) {
        List<ProductInfoVO> notTanpuProductInfolist = this.productService.getNotTanpuProductInfolist(req);
        return CommonResp.success(notTanpuProductInfolist);
    }

    @Override
    public CommonResp<DynamicRetreatVO> getDynamicRetreatInfo(String id) {
        return CommonResp.success(this.productService.getDynamicRetreat(id));
    }

    @Override
    public CommonResp<FundDateResp> getFundDateByFundIdInfo(String fundId) {
        return CommonResp.success(this.productService.getFundDateByFundId(fundId));
    }


}
