package com.tanpu.fund.service;

import java.util.Date;
import java.util.List;

/**
 * 基金相关统计
 */
public interface FundCountService {
    /**
     * 计算区间收益率（批量）
     *
     * 批量计算 - 根据披露频率分类
     *
     * @param proDisFre 披露类型
     * @param startDate 区间开始时间
     * @param endDate 区间结束时间
     * @param fundList 基金ID集合
     *
     * @return
     */
    void sectionProfitRate(int proDisFre, Date startDate, Date endDate, List<String> fundList);

    /**
     * 计算区间收益率（单只基金）
     *
     * @param proDisFre 披露类型
     * @param startDate 区间开始时间
     * @param endDate 区间结束时间
     * @param fundId 基金ID
     */
    void sectionProfitRate(int proDisFre, Date startDate, Date endDate, String fundId);

    /**
     * 计算年化波动率（即标准差）
     *
     * @param fundId 基金ID
     * @param proDisFre 基金披露类型
     */
    void annualRate(String fundId, int proDisFre);

    /**
     * 计算最大回撤
     *
     * @param fundId
     */
    void retracement(String fundId);
}
