package com.tanpu.fund.service.impl;

import com.tanpu.common.enums.Constant;
import com.tanpu.fund.api.model.FundInfoVO;
import com.tanpu.fund.api.model.FundRatioVO;
import com.tanpu.fund.api.model.resp.TrackPositionVO;
import com.tanpu.fund.entity.generator.*;
import com.tanpu.fund.enums.ProTypeEnums;
import com.tanpu.fund.service.FundHistoryHoldService;
import com.tanpu.common.utils.BigDecimalUtil;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class FundHistoryHoldServiceImpl implements FundHistoryHoldService {
    @Resource
    private FundCombineInfoMapper fundCombineInfoMapper;

    @Resource
    private FundInfoMapper fundInfoMapper;

    @Override
    public List<Long> getTrackPositionDates(String id) {
        List<Long> list = null;

        FundCombineInfoExample example = new FundCombineInfoExample();
        example.createCriteria().andPFundIdEqualTo(id).andDeleteTagEqualTo(Constant.ZERO_NUM);
        example.setOrderByClause("change_date desc");
        List<FundCombineInfo> fundCombineInfos = fundCombineInfoMapper.selectByExample(example);

        Set<Long> collect = fundCombineInfos.stream().filter(f -> f.getChangeDate() != null).map(f -> f.getChangeDate().getTime()).collect(Collectors.toSet());

        if (!CollectionUtils.isEmpty(collect)) {
            list = new ArrayList<>(collect);
            // 由大到小排序
            list.sort(Comparator.comparingLong(Long::longValue).reversed());
        }

        return list;
    }

    @Override
    public TrackPositionVO getTrackPositionDetail(String id, Long date) {
        TrackPositionVO trackPositionVO = new TrackPositionVO();
        trackPositionVO.setTime(date);

        //查询组合基金
        FundCombineInfoExample example = new FundCombineInfoExample();
        example.createCriteria()
                .andPFundIdEqualTo(id)
                .andChangeDateEqualTo(new Date(date))
                .andDeleteTagEqualTo(Constant.ZERO_NUM);
        List<FundCombineInfo> fundCombineInfos = fundCombineInfoMapper.selectByExample(example);

        //基金+占比
        Map<String, BigDecimal> map = fundCombineInfos.stream().collect(Collectors.toMap(FundCombineInfo::getFundId, FundCombineInfo::getProportionRatio));

        FundInfoExample fundInfoExample = new FundInfoExample();
        fundInfoExample.createCriteria().andIdIn(new ArrayList<>(map.keySet())).andDeleteTagEqualTo(Constant.ZERO_NUM);
        List<FundInfo> fundInfoList = fundInfoMapper.selectByExample(fundInfoExample);

        //按类型统计基金占比
        {
            //按类型分类
            Map<Integer, List<String>> typeMap = fundInfoList.stream().collect(Collectors.toMap(FundInfo::getFundType, item -> {
                List<String> idList = new ArrayList<>();
                idList.add(item.getId());
                return idList;
            }, (List<String> list1, List<String> list2) -> {
                list1.addAll(list2);
                return list1;
            }));

            {
                //计算每种类型的占比
                List<FundRatioVO> fundRatios = new ArrayList<>();
                for (Integer type : typeMap.keySet()) {
                    FundRatioVO p = new FundRatioVO();
                    p.setBonusType(type);

                    List<String> list = typeMap.get(type);
                    double sum = list.stream().mapToDouble(fundId -> map.get(fundId).doubleValue()).sum();
                    p.setRatio(String.valueOf(new BigDecimal(sum).setScale(2, BigDecimal.ROUND_HALF_UP)));

                    fundRatios.add(p);
                }

                trackPositionVO.setFundRatios(fundRatios);
            }
        }

        //每只基金的占比
        {
            Map<String, FundInfo> fundInfoMap = fundInfoList.stream().collect(Collectors.toMap(FundInfo::getId, item -> item));

            List<FundInfoVO> resList = fundCombineInfos.stream().map(item -> {
                FundInfo fundInfo = fundInfoMap.get(item.getFundId());

                FundInfoVO p = new FundInfoVO();
                p.setFundId(item.getFundId());
                p.setBonusTypeName(ProTypeEnums.getName(fundInfo.getFundType()));
                p.setBonusType(fundInfo.getFundType());
                p.setFundName(fundInfo.getFundShortName());
                p.setRatio(BigDecimalUtil.multiply100(item.getProportionRatio()));
                p.setRatioTransfer(BigDecimalUtil.multiply100(item.getCompareLast()));
                return p;
            }).collect(Collectors.toList());

            trackPositionVO.setFundInfos(resList);
        }

        return trackPositionVO;
    }
}
