package com.tanpu.fund.service.impl;

import cn.hutool.core.date.DateUtil;
import com.google.common.collect.Lists;
import com.tanpu.common.enums.BizEnums;
import com.tanpu.common.enums.Constant;
import com.tanpu.common.enums.fund.FundFileTypeEnum;
import com.tanpu.common.model.product.resp.*;
import com.tanpu.common.utils.BigDecimalUtil;
import com.tanpu.fund.entity.generator.*;
import com.tanpu.fund.mapper.generator.*;
import com.tanpu.fund.mapper.generator.custom.FundInfoCustomMapper;
import com.tanpu.fund.service.ProductOrderService;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.function.BinaryOperator;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author: zhoupeng
 * @email: zhoupeng_08@163.com
 */
@Service
public class ProductOrderServiceImpl implements ProductOrderService, Constant {

    @Value("${file.preview.path}")
    private String suffix;

    @Resource
    private FundInfoMapper fundInfoMapper;

    @Resource
    private FundCountMapper fundCountMapper;

    @Resource
    private FundBankMapper fundBankMapper;

    @Resource
    private FundRateMappingMapper fundRateMappingMapper;
    @Resource
    private ContractTemplateManageMapper contractTemplateManageMapper;
    @Resource
    private FundFileRecordMapper fundFileRecordMapper;

    @Resource
    private FundInfoCustomMapper fundInfoCustomMapper;

    @Resource
    private FundDateMapper fundDateMapper;

    @Override
    public List<FundInfoResp> getFundInfos(List<String> list) {
        FundInfoExample example = new FundInfoExample();
        example.createCriteria().andIdIn(list).andDeleteTagEqualTo(Constant.ZERO_NUM);

        return getFundInfoResps(list, example);
    }

    @NotNull
    private List<FundInfoResp> getFundInfoResps(List<String> list, FundInfoExample example) {
        FundBankExample example1 = new FundBankExample();
        example1.createCriteria().andFundIdIn(list).andDeleteTagEqualTo(Constant.ZERO_NUM);
        Map<String, FundBank> fundBankMap = this.fundBankMapper.selectByExample(example1).stream().collect(Collectors.toMap(FundBank::getFundId, f -> f));
        FundCountExample fundCountExample = new FundCountExample();
        fundCountExample.createCriteria().andFundIdIn(list).andDeleteTagEqualTo(Constant.ZERO_NUM);
        List<FundCount> fundCounts = fundCountMapper.selectByExample(fundCountExample);
        Map<String, FundCount> fundCountMap;
        if (CollectionUtils.isNotEmpty(fundCounts)) {
            fundCountMap = fundCounts.stream().collect(Collectors.toMap(FundCount::getFundId, c -> c));
        } else {
            fundCountMap = null;
        }

        // 基金基本信息
        FundRateMappingExample example2 = new FundRateMappingExample();
        example2.createCriteria().andFundIdIn(list).andDeleteTagEqualTo(Constant.ZERO_NUM);
        Map<String, FundRateMapping> rateMappingMap = this.fundRateMappingMapper.selectByExample(example2).stream().collect(Collectors.toMap(FundRateMapping::getFundId, f -> f));

        // 合同及指南管理
        ContractTemplateManageExample contractTemplateManageExample = new ContractTemplateManageExample();
        contractTemplateManageExample.createCriteria()
                .andFundIdIn(list).andDeleteTagEqualTo(BizEnums.DeleteTag.tag_init);
        Map<String, ContractTemplateManage> contractTemplateManageMap =
                contractTemplateManageMapper.selectByExample(contractTemplateManageExample)
                        .stream().collect(Collectors.toMap(ContractTemplateManage::getFundId, v -> v, (k1, k2) -> k1));

        // 开放日 赎回日
        Map<String, Long> fundDateMap = fundDateHandler(list);


        return this.fundInfoMapper.selectByExample(example).stream().map(f -> {
            FundInfoResp resp = new FundInfoResp();
            BeanUtils.copyProperties(f, resp);

            FundBank fundBank = fundBankMap.get(f.getId());
            if (fundBank != null) {
                resp.setAccountName(fundBank.getAccountName());
                resp.setBankName(fundBank.getBankName());
                resp.setFundAccount(fundBank.getFundAccount());
            }

            FundRateMapping fundRateMapping = rateMappingMap.get(f.getId());
            if (fundRateMapping != null) {
                resp.setMinInvestmentShare(BigDecimalUtil.toString(fundRateMapping.getMinInvestmentShare()));
                resp.setSubsequentInvestmentShare(BigDecimalUtil.toString(fundRateMapping.getSubsequentInvestmentShare()));
            }

            /*if (f.getApplyDay() != null) {
                resp.setApplyDay(f.getApplyDay().getTime());
            }
            if (f.getRedeemDay() != null) {
                resp.setRedeemDay(f.getRedeemDay().getTime());
            }*/
            if (fundDateMap.containsKey(f.getId() + ONE_NUM)) {
                resp.setApplyDay(fundDateMap.get(f.getId() + ONE_NUM));
            }
            if (fundDateMap.containsKey(f.getId() + THREE_NUM)) {
                resp.setRedeemDay(fundDateMap.get(f.getId() + THREE_NUM));
            }

            if (fundCountMap != null) {
                FundCount fundCount = fundCountMap.get(f.getId());
                BigDecimal netNav = fundCount.getNetNav();
                resp.setNetNav(BigDecimalUtil.toString(netNav, 4));
                resp.setNet(Net.builder().netDate(fundCount.getEndDate().getTime()).netValue(BigDecimalUtil.toString(netNav, 4)).build());
            }

            ContractTemplateManage t = contractTemplateManageMap.get(f.getId());
            if (t != null) {
                resp.setExpenseSubscription(t.getExpenseSubscription());
                resp.setExpenseManage(t.getExpenseManage());
                resp.setExpenseAchievement(t.getExpenseAchievement());
                resp.setIfaSubscription(t.getIfaSubscription());
                resp.setIfaManage(t.getIfaManage());
                resp.setIfaAchievement(t.getIfaAchievement());
            } else {
                resp.setExpenseSubscription(StringUtils.EMPTY);
                resp.setExpenseManage(StringUtils.EMPTY);
                resp.setExpenseAchievement(StringUtils.EMPTY);
                resp.setIfaSubscription(StringUtils.EMPTY);
                resp.setIfaManage(StringUtils.EMPTY);
                resp.setIfaAchievement(StringUtils.EMPTY);
            }

            return resp;
        }).collect(Collectors.toList());
    }

    @Override
    public List<FundInfoResp> getNewFundInfos(List<String> list) {
        FundInfoExample example = new FundInfoExample();
        example.createCriteria().andIdIn(list);

        return getFundInfoResps(list, example);
    }

    private Map<String, Long> fundDateHandler(List<String> list) {
        FundDateExample example3 = new FundDateExample();
        example3.createCriteria().andFundIdIn(list).andDateGreaterThanOrEqualTo(DateUtil.beginOfDay(new Date()));
        List<FundDate> fundDates = this.fundDateMapper.selectByExample(example3);
        Map<String, Long> fundDateMap;
        if (CollectionUtils.isEmpty(fundDates)) {
            return new HashMap<>(0);
        }
        Map<String, List<FundDate>> fundIdsMap = fundDates.stream().collect(Collectors.groupingBy(FundDate::getFundId));

        fundDateMap = new HashMap<>(fundIdsMap.keySet().size());
        fundIdsMap.forEach((fundId, fundIds) -> {
            Map<Integer, FundDate> dateMap = fundIds.stream()
                    .collect(Collectors.toMap(FundDate::getType,
                            Function.identity(),
                            BinaryOperator.minBy(Comparator.comparingLong(l -> l.getDate().getTime()))));
            dateMap.forEach((type, fundDate) -> {
                if (type.equals(ONE_NUM) || type.equals(TWO_NUM)) {
                    if (fundDateMap.containsKey(fundId + ONE_NUM) && fundDateMap.get(fundId + ONE_NUM) > fundDate.getDate().getTime()) {
                        fundDateMap.put(fundId + ONE_NUM, fundDate.getDate().getTime());
                    } else {
                        fundDateMap.put(fundId + ONE_NUM, fundDate.getDate().getTime());
                    }
                } else if (type.equals(THREE_NUM)) {
                    fundDateMap.put(fundId + type, fundDate.getDate().getTime());
                }
            });
        });
        return fundDateMap;
    }

    /**
     * 获取主和子合同模板
     *
     * @return
     */
    @Override
    public List<AttachmentInfoVO> getContractTemplate() {
        FundFileRecordExample example = new FundFileRecordExample();
        example.createCriteria()
                .andTypeIn(Lists.newArrayList(
                        FundFileTypeEnum.MAIN_CONTRACT_TEMPLATE.getCode(), FundFileTypeEnum.SUB_CONTRACT_TEMPLATE.getCode()))
                .andDeleteTagEqualTo(BizEnums.DeleteTag.tag_init);
        example.setOrderByClause("create_time desc");

        List<FundFileRecord> fundFileRecords = this.fundFileRecordMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(fundFileRecords)) {
            return Lists.newArrayListWithCapacity(0);
        }

        ArrayList<AttachmentInfoVO> list = Lists.newArrayListWithExpectedSize(2);
        Optional<FundFileRecord> main = fundFileRecords.stream()
                .filter(f -> f.getType() == FundFileTypeEnum.MAIN_CONTRACT_TEMPLATE.getCode()).findFirst();
        if (main.isPresent()) {
            AttachmentInfoVO vo = new AttachmentInfoVO();
            BeanUtils.copyProperties(main.get(), vo);
            list.add(vo);
        }

        Optional<FundFileRecord> sub = fundFileRecords.stream()
                .filter(f -> f.getType() == FundFileTypeEnum.SUB_CONTRACT_TEMPLATE.getCode()).findFirst();
        if (sub.isPresent()) {
            AttachmentInfoVO vo = new AttachmentInfoVO();
            BeanUtils.copyProperties(sub.get(), vo);
            list.add(vo);
        }
        return list;
    }

    @Override
    public List<NetBigDecimal> getFundLastNet(List<String> list) {
        List<FundNav> fundInfoLastNet = fundInfoCustomMapper.getFundInfoLastNet(list);

        List<NetBigDecimal> resList;
        if (CollectionUtils.isNotEmpty(fundInfoLastNet)) {
            resList = fundInfoLastNet.stream().map(item -> NetBigDecimal.builder().netDate(item.getPriceDate().getTime()).netValue(item.getNav()).build()).collect(Collectors.toList());
        } else {
            resList = Lists.newArrayListWithCapacity(0);
        }

        return resList;
    }

    @Override
    public List<FilePreviewResp> getFilePreview(List<String> list) {
        if (CollectionUtils.isEmpty(list)) {
            return new ArrayList<>(0);
        }
        return this.fundInfoCustomMapper.getFilePreviewUrl(list.stream().collect(Collectors.joining("','", "'", "'")))
                .stream().peek(f -> f.setPreviewUrl(suffix + f.getPreviewUrl().substring(f.getPreviewUrl().lastIndexOf("/") + 1, f.getPreviewUrl().lastIndexOf(".")))).collect(Collectors.toList());
    }
}
